@@ -606,7 +606,7 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 		    index = (int)
 			XSLT_RUNTIME_EXTRA(ctxt, select->indexExtra);
 		    list = (xmlXPathObjectPtr)
-			XSLT_RUNTIME_EXTRA(ctxt, select->lenExtra);
+			XSLT_RUNTIME_EXTRA_LST(ctxt, select->lenExtra);
 		    
 		    doc = node->doc;
 		    if ((list == NULL) || (prevdoc != doc)) {
@@ -644,17 +644,21 @@ xsltTestCompMatch(xsltTransformContextPtr ctxt, xsltCompMatchPtr comp,
 				xmlXPathFreeObject(list);
 			    list = newlist;
 
-			    XSLT_RUNTIME_EXTRA(ctxt, select->lenExtra) =
+			    XSLT_RUNTIME_EXTRA_LST(ctxt, select->lenExtra) =
 				(void *) list;
 			    XSLT_RUNTIME_EXTRA(ctxt, select->previousExtra) =
 				(void *) doc;
 			    XSLT_RUNTIME_EXTRA_FREE(ctxt, select->lenExtra) =
 				(xmlFreeFunc) xmlXPathFreeObject;
-			}
+			} else
+			    list = newlist;
 		    }
 		    if ((list->nodesetval == NULL) ||
-			(list->nodesetval->nodeNr <= 0))
+			(list->nodesetval->nodeNr <= 0)) {
+			if (nocache == 1)
+			    xmlXPathFreeObject(list);
 			return(0);
+		    }
 		    /* TODO: store the index and use it for the scan */
 		    if (index == 0) {
 			for (j = 0;j < list->nodesetval->nodeNr;j++) {
