@@ -316,7 +316,7 @@ xsltSortComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 	    xsltPrintErrorContext(NULL, style, inst);
 	    xsltGenericError(xsltGenericErrorContext,
 		 "xsltSortComp: no support for data-type = %s\n", comp->stype);
-	    comp->number = -1;
+	    comp->number = 0; /* use default */
 	    style->warnings++;
 	}
     }
@@ -332,7 +332,7 @@ xsltSortComp(xsltStylesheetPtr style, xmlNodePtr inst) {
 	    xsltPrintErrorContext(NULL, style, inst);
 	    xsltGenericError(xsltGenericErrorContext,
 		 "xsltSortComp: invalid value %s for order\n", comp->order);
-	    comp->descending = -1;
+	    comp->descending = 0; /* use default */
 	    style->warnings++;
 	}
     }
@@ -453,7 +453,15 @@ xsltElementComp(xsltStylesheetPtr style, xmlNodePtr inst) {
     comp->ns = xsltEvalStaticAttrValueTemplate(style, inst,
 			 (const xmlChar *)"namespace",
 			 XSLT_NAMESPACE, &comp->has_ns);
+    if (comp->has_ns == 0) {
+	xmlNsPtr defaultNs;
 
+	defaultNs = xmlSearchNs(inst->doc, inst, NULL);
+	if (defaultNs != NULL) {
+	    comp->ns = xmlStrdup(defaultNs->href);
+	    comp->has_ns = 1;
+	}
+    }
     comp->use = xsltEvalStaticAttrValueTemplate(style, inst,
 		       (const xmlChar *)"use-attribute-sets",
 		       XSLT_NAMESPACE, &comp->has_use);
